import os
from typing import Optional

import hvac

from mind_castle.secret_store_base import SecretStoreBase


class HashiCorpVaultSecretStore(SecretStoreBase):
    """
    Uses HashiCorp Vault to store secrets.
    """

    store_type = "hashicorpvault"
    required_config = [
        "MIND_CASTLE_VAULT_HOST",
        "MIND_CASTLE_VAULT_TOKEN",
    ]

    def __init__(self):
        self.client = hvac.Client(
            url=f"{os.environ.get('MIND_CASTLE_VAULT_SCHEMA', 'http')}://{os.environ.get('MIND_CASTLE_VAULT_HOST')}",
            token=f"{os.environ.get('MIND_CASTLE_VAULT_TOKEN')}",
        )

    def get_secret(self, key: str, default: str = None) -> Optional[str]:
        response = self.client.secrets.kv.read_secret_version(path=key)
        return response["data"]["data"].get("secret_value", default)

    def put_secret(self, value: str) -> dict:
        key = self.get_secret_key()
        self.client.secrets.kv.v2.create_or_update_secret(
            path=key, secret=dict(secret_value=value)
        )  # Value has to be a dict so just make a 'secret_value' key
        return {"secret_type": self.store_type, "key": key}



import iterfzf
import typer
import json
from sqlalchemy import create_engine, select, table, inspect, func, and_
from sqlalchemy.orm import Session
from sqlalchemy.ext.automap import automap_base
from rich import print
from rich.prompt import Prompt
from rich.table import Table as RichTable
from rich.panel import Panel
from rich.progress import track

from mind_castle.stores import put_secret


def main(target_table: str = None, column: str = None, dry_run: bool = True, secret_type: str = "json"):
    print("Creating DB engine..")
    # engine = create_engine("postgresql://postgres:postgres@localhost:15432/mindsdb?sslmode=disable")

    # reflect the tables
    Base = automap_base()
    Base.prepare(autoload_with=engine)

    with Session(engine) as session:
        # None of the changes below will be committed to the DB unless we run session.commit()

        # Get the user's selection of table and column
        insp = inspect(engine)
        tables = insp.get_table_names()
        target_table = iterfzf.iterfzf(tables, prompt="Target table > ")

        # Create a auto-mapped class for the target table
        TargetTable = getattr(Base.classes, target_table)

        columns = TargetTable.__table__.columns.keys()
        target_column = iterfzf.iterfzf(columns, prompt="Target column > ")

        

        # Analyse and plan what to do. See how many rows we'll have to edit
        # We're not going to change any rows that have an empty value in the target column so don't count them
        filter_exclusions = and_(getattr(TargetTable, target_column) != "{}", getattr(TargetTable, target_column) != "")
        stmt = select(func.count()).select_from(TargetTable).where(filter_exclusions)
        rows_to_edit = session.execute(stmt).scalar_one()

        # Warn the user of what's about to happen
        print(Panel("[bold underline red]YOU'RE ABOUT TO MODIFY YOUR DATABASE[/bold underline red]\n\nIf you continue and don't have a backup of your DB, you should feel bad.", expand=False))

        print(f"- There are {rows_to_edit} rows about to be altered in the '{target_table}' table.")
        print(f"- We will be modifying the '{target_column}' column.")
        print("- Only JSON columns are supported for now")
        print("- This could take a while.")
        print("- If any error occurs at any point, we will bail out and make no changes.")
        print("- You will get another chance to bail out before changes are committed.")

        cont = typer.confirm("Continue?")
        if not cont:
            raise typer.Abort()

        rows = session.query(TargetTable).filter(filter_exclusions).all()
        secrets_stored = 0
        for row in track(rows):
            data_str = getattr(row, target_column)
            data = json.loads(data_str)  # Check the data is valid JSON
            if not dry_run:
                secret_def = put_secret(data_str, secret_type)
                setattr(row, target_column, json.dumps(secret_def))
            secrets_stored += 1
            
        if secrets_stored != rows_to_edit:
            print(f"ERROR: Number of modified rows ({secrets_stored}) does not match expected ({rows_to_edit}). Is the database being written to?")
            raise typer.Abort()
        
        if dry_run:
            print("Dry run completed successfully.")
            raise typer.Exit()
        
        cont = typer.confirm("Changes have been queued successfully. Commit them?")
        if not cont:
            raise typer.Abort()

        session.commit()


if __name__ == "__main__":
    typer.run(main)



import os
from typing import Optional

import boto3

from mind_castle.secret_store_base import SecretStoreBase


class AWSSecretsManagerSecretStore(SecretStoreBase):
    """
    Uses AWS Secrets Manager to store secrets.
    This assumes you have credentials in the environment or in the default AWS credentials file.
    """

    store_type = "awssecretsmanager"
    required_config = [
        "MIND_CASTLE_AWS_REGION",
        "MIND_CASTLE_AWS_ACCESS_KEY_ID",
        "MIND_CASTLE_AWS_SECRET_ACCESS_KEY",
    ]

    def __init__(self):
        # TODO: Use this region param as a test for how to pass args to a secret store init
        self.client = boto3.client(
            "secretsmanager",
            region_name=os.environ.get("MIND_CASTLE_AWS_REGION"),
            aws_access_key_id=os.environ.get("MIND_CASTLE_AWS_ACCESS_KEY_ID"),
            aws_secret_access_key=os.environ.get("MIND_CASTLE_AWS_SECRET_ACCESS_KEY"),
        )

    def get_secret(self, key: str, default: str = None) -> Optional[str]:
        response = self.client.get_secret_value(SecretId=key)
        return response.get("SecretString", default)

    def put_secret(self, value: str) -> dict:
        key = self.get_secret_key()
        # Response is always a success. This throws an exception if it fails
        response = self.client.create_secret(
            Name=key,
            ClientRequestToken=key,
            SecretString=value,
            # Other details we could add:
            # Description='string',
            # KmsKeyId='string',
            # SecretBinary=b'bytes',
            # Tags=[
            #     {
            #         'Key': 'string',
            #         'Value': 'string'
            #     },
            # ],
            # AddReplicaRegions=[
            #     {
            #         'Region': 'string',
            #         'KmsKeyId': 'string'
            #     },
            # ],
            # ForceOverwriteReplicaSecret=True|False
        )

        # ARN is not needed but returned just in case. we could also add tags etc to this response
        return {"secret_type": self.store_type, "key": key, "arn": response.get("ARN")}



from mind_castle.stores.memory import MemorySecretStore


def test_put_secret():
    secret_store = MemorySecretStore()
    response = secret_store.put_secret("some_secret_value")

    # Just read the memory from this implementation
    assert secret_store.secrets[response["key"]] == "some_secret_value"


def test_get_secret():
    secret_store = MemorySecretStore()
    response = secret_store.put_secret("some_secret_value")

    assert secret_store.get_secret(response["key"]) == "some_secret_value"



from unittest.mock import ANY

from mind_castle.stores.vault import HashiCorpVaultSecretStore


def test_put_secret(mocker):
    secret_store = HashiCorpVaultSecretStore()
    mocker.patch.object(
        secret_store.client.secrets.kv.v2, "create_or_update_secret", return_value={}
    )
    response = secret_store.put_secret("some_secret_value")

    assert secret_store.client.secrets.kv.v2.create_or_update_secret.called_once_with(
        secret=dict(path=ANY, secret_value="some_secret_value")
    )
    assert response.get("secret_type") == "hashicorpvault"
    assert response.get("key") is not None


def test_get_secret(mocker):
    mock_vault_response = {
        "data": {
            "data": {
                "secret_value": "some_secret_value",
            }
        }
    }

    secret_store = HashiCorpVaultSecretStore()
    mocker.patch.object(
        secret_store.client.secrets.kv,
        "read_secret_version",
        return_value=mock_vault_response,
    )
    response = secret_store.get_secret("some_secret_key2")

    assert secret_store.client.secrets.kv.read_secret_version.calle_once_with(
        path="some_secret_key2"
    )
    assert response == "some_secret_value"



from mind_castle import stores  # noqa: F401
from mind_castle.secret_store_base import SecretStoreBase

print("\nMind-Castle - Shhhhh")
print("====================")
print("Available secret stores:\n")

longest_name = max(
    [len(store.store_type) for store in SecretStoreBase.__subclasses__()]
)

for store in SecretStoreBase.__subclasses__():
    print(
        f"{store.store_type.ljust(longest_name)} - Required env vars: {store.required_config}"
    )



import uuid
from abc import ABC, abstractmethod
from typing import Optional


class SecretStoreBase(ABC):
    """This is an abstract class that defines the interface for a secret store."""

    store_type = "base"
    required_config = []

    @abstractmethod
    def get_secret(self, key: str, default: str = None) -> Optional[str]:
        raise NotImplementedError()

    @abstractmethod
    def put_secret(self, value: str) -> dict:
        raise NotImplementedError()

    def get_secret_key(self) -> str:
        """
        Generates a key based on this machine's hardware and the time.
        https://docs.python.org/3/library/uuid.html#uuid.uuid1
        """

        return str(uuid.uuid1())



import json
import logging
import os
from typing import Optional

from mind_castle.secret_store_base import SecretStoreBase

logger = logging.getLogger(__name__)

# DO NOT USE THIS STORE IN PRODUCTION
# It will store your secrets IN PLAIN TEXT in a JSON file


class JsonSecretStore(SecretStoreBase):
    store_type = "json"
    filename = "secrets.json"

    def __init__(self):
        logger.warning(
            "You are using the JSON secret store. DO NOT USE THIS IN PRODUCTION."
        )
        if not os.path.exists(self.filename):
            with open(self.filename, "w") as f:
                f.write("{}")

    def get_secret(self, key: str, default: str = None) -> Optional[str]:
        with open(self.filename, "r") as f:
            secrets = json.load(f)
        return secrets.get(key, default)

    def put_secret(self, value: str) -> dict:
        key = self.get_secret_key()
        with open(self.filename, "r") as f:
            secrets = json.load(f)
            secrets[key] = value

        with open(self.filename, "w") as f:
            json.dump(secrets, f)
        return {"secret_type": self.store_type, "key": key}



import json
import logging

from sqlalchemy import types

from mind_castle.stores import get_secret, put_secret

logger = logging.getLogger(__name__)
logger.level = logging.DEBUG


class SecretData(types.TypeDecorator):
    """A sqlalchemy field type that stores data in a secret store."""

    impl = types.JSON  # The base data type for this field
    cache_ok = True

    def __init__(self, store_type: str, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.secret_type = store_type

    def process_bind_param(self, value, dialect):
        # Don't waste time storing empty values
        if value is None or value == "" or value == {} or value == []:
            return value

        # This might not be a dict or list, but let's json dump everything to be consistent
        stringValue = json.dumps(value)

        secret_params = put_secret(stringValue, self.secret_type)
        logger.debug(f"Stored secret with: {secret_params}")
        return secret_params

    def process_result_value(self, value, dialect):
        if not isinstance(value, dict) or value.get("secret_type") is None:
            logger.debug(
                f"No secret type found in '{value}', must be a plantext value."
            )
            return value

        logger.debug(f"Restoring {value} from a secret.")
        return json.loads(get_secret(value))



import glob
import logging
import os
from os.path import basename, dirname, isfile, join

from mind_castle.secret_store_base import SecretStoreBase

modules = glob.glob(join(dirname(__file__), "*.py"))
__all__ = [
    basename(f)[:-3] for f in modules if isfile(f) and not f.endswith("__init__.py")
]

from . import *  # noqa: F401, F403, E402, isort:skip

logger = logging.getLogger(__name__)
logger.level = logging.DEBUG

# Get all env vars with our prefix
env_vars = {k: v for k, v in os.environ.items() if k.startswith("MIND_CASTLE_")}

stores = {}

# Populate 'stores' dict with instantiated stores
for subclass in SecretStoreBase.__subclasses__():
    if all(e in env_vars.keys() for e in subclass.required_config):
        stores[subclass.store_type] = subclass()
        secret_store = subclass()
        logger.debug(f"Configured secret store '{subclass.store_type}'")
    elif any(e in env_vars.keys() for e in subclass.required_config):
        logger.warning(
            f"Missing required config for store '{subclass.store_type}'. Requires: {subclass.required_config}"
        )


def get_secret(data: dict, default: str = None) -> str:
    secret_type = data.get("secret_type")
    if secret_type is None:
        raise ValueError("No secret_type provided in data")

    secret_store = stores.get(secret_type)
    if secret_store is None:
        raise ValueError(
            f"Trying to retrieve secret from unknown/unconfigured store '{secret_type}'"
        )
    # TODO: handle 'key' key missing
    return secret_store.get_secret(data.get("key"), default)


def put_secret(value: str, secret_type: str) -> dict:
    secret_store = stores.get(secret_type)
    if secret_store is None:
        raise ValueError(
            f"Trying to put secret to unknown/unconfigured store '{secret_type}'"
        )

    return secret_store.put_secret(value)



from moto import mock_aws

from mind_castle.stores.aws import AWSSecretsManagerSecretStore


@mock_aws
def test_put_secret(mocker):
    secret_store = AWSSecretsManagerSecretStore()
    response = secret_store.put_secret("some_secret_value")

    # Read the secret from AWS directly to check
    client = secret_store.client  # Use the same auth etc as the store
    boto_response = client.get_secret_value(SecretId=response["key"])
    assert boto_response["SecretString"] == "some_secret_value"


@mock_aws
def test_get_secret():
    secret_store = AWSSecretsManagerSecretStore()
    # Add secret directly to AWS
    client = secret_store.client  # Use the same auth etc as the store
    client.create_secret(Name="some_secret_key", SecretString="some_secret_value")

    assert secret_store.get_secret("some_secret_key") == "some_secret_value"



import logging
from typing import Optional

from mind_castle.secret_store_base import SecretStoreBase

logger = logging.getLogger(__name__)


# DO NOT USE THIS STORE IN PRODUCTION
# It will store your secrets IN PLAIN TEXT in memory and
# they will be lost when the application is restarted


class MemorySecretStore(SecretStoreBase):
    """
    An in-memory secret store.
    This is not persistent and will be lost when the application is restarted.
    Use only for testing or development.
    """

    store_type = "memory"

    def __init__(self):
        logger.warning(
            "You are using the memory secret store. DO NOT USE THIS IN PRODUCTION."
        )
        self.secrets = {}

    def get_secret(self, key: str, default: str = None) -> Optional[str]:
        return self.secrets.get(key, default)

    def put_secret(self, value: str) -> dict:
        key = self.get_secret_key()
        self.secrets[key] = value
        return {"secret_type": self.store_type, "key": key}



import json

from mind_castle.stores.json import JsonSecretStore


def test_put_secret():
    secret_store = JsonSecretStore()
    response = secret_store.put_secret("some_secret_value")

    # Read the json file and check that the secret was written
    with open(secret_store.filename, "r") as f:
        secrets = json.load(f)
    assert secrets[response["key"]] == "some_secret_value"


def test_get_secret():
    secret_store = JsonSecretStore()
    response = secret_store.put_secret("some_secret_value")

    assert secret_store.get_secret(response["key"]) == "some_secret_value"
